from flask import Flask, request, Response
from flask_restful import Resource, Api

from mmif import Mmif


class Restifier(object):
    def __init__(self, app_instance, loopback=False, port=5000, debug=True):
        super().__init__()
        self.cla = app_instance
        self.import_name = app_instance.__class__.__name__
        self.flask_app = Flask(self.import_name)
        self.host = 'localhost' if loopback else '0.0.0.0'
        self.port = port
        self.debug = debug
        api = Api(self.flask_app)
        api.add_resource(ClamsRestfulApi, '/',
                         resource_class_args=[self.cla])

    def run(self):
        self.flask_app.run(host=self.host,
                           port=self.port,
                           debug=self.debug)

    def test_client(self):
        return self.flask_app.test_client()


class ClamsRestfulApi(Resource):

    def __init__(self, cla_instance):
        super().__init__()
        self.cla = cla_instance

    @staticmethod
    def json_to_response(json_str: str, status=200):
        return Response(response=json_str, status=status, mimetype='application/json')

    def get(self):
        return self.json_to_response(self.cla.appmetadata())

    def post(self):
        try:
            accept = self.cla.sniff(Mmif(request.get_data()))
            return Response(status=200) if accept else Response(status=406)
        except Exception as e:
            return Response(status=406, response=str(e))

    def put(self):
        return self.json_to_response(self.cla.annotate(Mmif(request.get_data())))




from typing import Dict, Type

from clams.serve import *
from clams.serve import __all__ as serve_all
from clams.restify import Restifier
from clams.source import PipelineSource, SourceCli
from clams.utils import Cli


__all__ = ['Restifier', 'PipelineSource'] + serve_all


CLIS: Dict[str, Type[Cli]] = {
    'source': SourceCli
}


def cli():
    import argparse
    import sys
    parser = argparse.ArgumentParser()
    parser.add_argument('verb', type=str, choices=['source'])
    args = parser.parse_args(sys.argv[1:2])
    if args.verb:
        CLIS[args.verb]().run()



from abc import ABC, abstractmethod
import json


__all__ = ['ClamsApp']


class ClamsApp(ABC):
    def __init__(self):
        # TODO (krim @ 10/9/20): eventually we might end up with a python class
        # for this metadata (with a JSON schema)
        self.metadata: dict = self.setupmetadata()
        super().__init__()

    def appmetadata(self):
        # TODO (krim @ 10/9/20): when self.metadata is no longer a `dict`
        # this method might needs to be changed to properly serialize input
        return json.dumps(self.metadata)

    @abstractmethod
    def setupmetadata(self) -> dict:
        raise NotImplementedError()

    @abstractmethod
    def sniff(self, mmif) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def annotate(self, mmif) -> str:
        raise NotImplementedError()



import setuptools

with open("VERSION", 'r') as version_f: 
    version = version_f.read().strip()

with open('README.md') as readme:
    long_desc = readme.read()

with open('requirements.txt') as requirements:
    requires = requirements.readlines()

setuptools.setup(
    name="clams-python", 
    version=version,
    author="Brandeis Lab for Linguistics and Computation", 
    author_email="admin@clams.ai",
    description="A collection of APIs to develop CLAMS app for python", 
    long_description=long_desc,
    long_description_content_type="text/markdown",
    url="https://www.clams.ai",
    classifiers=[
    'Development Status :: 2 - Pre-Alpha',
    'Framework :: Flask',
    'Framework :: Pytest',
    'Intended Audience :: Developers ',
    'License :: OSI Approved :: Apache Software License',
    'Programming Language :: Python :: 3 :: Only',
    ],
    install_requires=requires,
    python_requires='>=3.6',
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': [
            'clams = clams.__init__:cli',
        ],
    },
)
